#-*- coding:utf-8 -*-
# 2020 FinanceData.KR http://financedata.kr fb.com/financedata

import io
import zipfile
import requests
import xml.etree.ElementTree as ET
import pandas as pd

# 3-1 상장기업 재무정보 (단일회사): api/fnlttSinglAcnt.json
# 3-2 상장기업 재무정보 (다중회사): api/fnlttMultiAcnt.json
def finstate(api_key, corp_code, bsns_year, reprt_code='11011'):
    url = 'https://opendart.fss.or.kr/api/'
    url += 'fnlttMultiAcnt.json' if ',' in corp_code else 'fnlttSinglAcnt.json'

    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
        'bsns_year':  bsns_year,   # 사업년도
        'reprt_code': reprt_code, # "11011": 사업보고서
    }

    r = requests.get(url, params=params)
    jo = r.json() 
    if 'list' not in jo:
        print(jo)
        print('전자공시의 재무데이터는 2015년 이후 데이터를 제공합니다') if bsns_year < 2015 else print()
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])

# 3-3 재무제표 원본파일(XBRL): api/fnlttXbrl.xml
def finstate_xml(api_key, rcp_no, save_as='finstate_xml.zip', reprt_code='11011'):
    url = 'https://opendart.fss.or.kr/api/fnlttXbrl.xml'
    params = {
        'crtfc_key': api_key,
        'rcept_no': rcp_no,
        'reprt_code': reprt_code, # "11013"=1분기보고서,  "11012"=반기보고서,  "11014"=3분기보고서, "11011"=사업보고서
    }
    r = requests.get(url, params=params)
    try:
        tree = ET.XML(r.content)
        status = tree.find('status').text,
        message = tree.find('message').text
        if status != '000':
            raise ValueError({'status': status, 'message': message})
    except ET.ParseError as e:
        pass

    with open(save_as, 'wb') as f:
        f.write(r.content)
    return True

# 3-4 단일회사 전체 재무제표: api/fnlttSinglAcntAll.json
def finstate_all(api_key, corp_code, bsns_year, reprt_code='11011', fs_div='CFS'):
    url = 'https://opendart.fss.or.kr/api/fnlttSinglAcntAll.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
        'bsns_year':  bsns_year,   # 사업년도
        'reprt_code': reprt_code, # "11011": 사업보고서
        'fs_div': fs_div, # 'CFS'=연결제무제표, 'OFS'=별도(개별)제무제표
    }

    r = requests.get(url, params=params)
    jo = r.json() 
    if 'list' not in jo:
        print(jo)
        print('전자공시의 재무데이터는 2015년 이후 데이터를 제공합니다') if bsns_year < 2015 else print()
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])

# 3-5 XBRL 표준계정과목체계(계정과목): api/xbrlTaxonomy.json
def xbrl_taxonomy(api_key, sj_div='BS1'):
    url = 'https://opendart.fss.or.kr/api/xbrlTaxonomy.json'
    params = {
        'crtfc_key': api_key,
        'sj_div': sj_div, # "CFS":연결재무제표, "OFS":재무제표
    }
    r = requests.get(url, params=params)
    jo = r.json() 
    if 'list' not in jo:
        print(jo)
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])



import requests
import json
import pandas as pd

# 대량보유 상황보고
def major_shareholders(api_key, corp_code):
    url = 'https://opendart.fss.or.kr/api/majorstock.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
    }

    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    if jo['status'] != '000' or 'list' not in jo:
        print(jo)
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])

# 임원ㆍ주요주주 소유보고
def major_shareholders_exec(api_key, corp_code):
    url = 'https://opendart.fss.or.kr/api/elestock.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
    }

    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    if jo['status'] != '000' or 'list' not in jo:
        print(jo)
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])



# dart.py
#-*- coding:utf-8 -*-
# 2020~2023 FinanceData.KR http://financedata.kr fb.com/financedata

import requests
import zipfile
import os
import io
import glob
import json
import pandas as pd
import warnings
import xml.etree.ElementTree as ET
from datetime import datetime
from . import dart_list
from . import dart_report
from . import dart_finstate
from . import dart_share
from . import dart_event
from . import dart_regstate
from . import dart_utils

class OpenDartReader():
    # init corp_codes (회사 고유번호 데이터)
    def __init__(self, api_key):
        # read and return document if exists
        self.corp_codes = dart_list.corp_codes(api_key)
        self.api_key = api_key
        
    # 1. 공시정보
    # 1-1. 공시정보 - 공시검색
    def list(self, corp=None, start=None, end=None, kind='', kind_detail='', final=True):
        '''
        DART 보고서의 목록을 DataFrame으로 반환
        * corp: 종목코드 (고유번호, 법인명도 가능)
        * start: 조회 시작일 (기본값: 1990-01-01')
        * end: 조회 종료일 (기본값: 당일)
        * kind: 보고서 종류:  A=정기공시, B=주요사항보고, C=발행공시, D=지분공시, E=기타공시, 
                                        F=외부감사관련, G=펀드공시, H=자산유동화, I=거래소공시, J=공정위공시
        * final: 최종보고서 여부 (기본값: True)
        '''
        if corp:
            corp_code = self.find_corp_code(corp)
            if not corp_code:
                raise ValueError(f'could not find "{corp}"')
        else:
            corp_code = ''
        return dart_list.list(self.api_key, corp_code, start, end, kind, kind_detail, final)

    # 1-2. 공시정보 - 기업개황
    def company(self, corp):
        corp_code = self.find_corp_code(corp)
        return dart_list.company(self.api_key, corp_code)

    def company_by_name(self, name):
        df = self.corp_codes[self.corp_codes['corp_name'].str.contains(name)]
        corp_code_list = list(df['corp_code'])
        return dart_list.company_by_name(self.api_key, corp_code_list)
    
    # 1-3. 공시정보 - 공시서류원본문서 (사업보고서)
    def document(self, rcp_no, cache=True):
        return dart_list.document(self.api_key, rcp_no, cache=cache)

    # 1-3. 공시정보 - 공시서류원본문서 전체 (사업보고서, 감사보고서)
    def document_all(self, rcp_no, cache=True):
        return dart_list.document_all(self.api_key, rcp_no, cache=cache)

    # 1-4. 공시정보 - 고유번호: corp(종목코드 혹은 회사이름) to 고유번호(corp_code)
    def find_corp_code(self, corp):
        if not corp.isdigit():
            df = self.corp_codes[self.corp_codes['corp_name'] == corp]
        elif corp.isdigit() and len(corp) == 6:
            df = self.corp_codes[self.corp_codes['stock_code'] == corp]
        else:
            df = self.corp_codes[self.corp_codes['corp_code'] == corp]
        return None if df.empty else df.iloc[0]['corp_code']

    # 2. 사업보고서 
    def report(self, corp, key_word, bsns_year, reprt_code='11011'):
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        return dart_report.report(self.api_key, corp_code, key_word, bsns_year, reprt_code)

    # 3. 상장기업 재무정보
    # 3-1. 상장기업 재무정보 (단일회사)
    # 3-2. 상장기업 재무정보 (다중회사)
    def finstate(self, corp, bsns_year, reprt_code='11011'):
        if ',' in corp:
            code_list = [self.find_corp_code(c.strip()) for c in corp.split(',')]
            corp_code = ','.join(code_list)
        else:
            corp_code = self.find_corp_code(corp)
        return dart_finstate.finstate(self.api_key, corp_code, bsns_year, reprt_code)

    # 3-3. 재무제표 원본파일(XBRL)
    def finstate_xml(self, rcp_no, save_as):
        return dart_finstate.finstate_xml(self.api_key, rcp_no, save_as=save_as)

    # 3-4. 단일회사 전체 재무제표
    def finstate_all(self, corp, bsns_year, reprt_code='11011', fs_div='CFS'):
        reprt_code_dict = {'11013':'1분기보고서', '11012':'반기보고서', '11014':'3분기보고서', '11011':'사업보고서'}
        fs_div_dict = {'CFS':'연결제무제표', 'OFS':'별도(개별)제무제표'}
        
        if reprt_code not in reprt_code_dict:
            raise ValueError(f'invalid reprt_code (use one of {reprt_code_dict}')
        if fs_div not in fs_div_dict:
            raise ValueError(f'invalid fs_div (use one of {fs_div_dict}')
        
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        print(f"reprt_code='{reprt_code}', fs_div='{fs_div}' ({reprt_code_dict[reprt_code]}, {fs_div_dict[fs_div]})'")
        return dart_finstate.finstate_all(self.api_key, corp_code, bsns_year, reprt_code=reprt_code, fs_div=fs_div)
        
    # 3-5. XBRL 표준계정과목체계(계정과목)
    def xbrl_taxonomy(self, sj_div):
        return dart_finstate.xbrl_taxonomy(self.api_key, sj_div=sj_div)

    # 4. 지분공시 
    # 4-1. 지분공시 - 대량보유 상황보고
    def major_shareholders(self, corp):
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        return dart_share.major_shareholders(self.api_key, corp_code)

    # 4-2. 지분공시 - 임원ㆍ주요주주 소유보고
    def major_shareholders_exec(self, corp):
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        return dart_share.major_shareholders_exec(self.api_key, corp_code)

    # 5. 주요사항보고서 
    def event(self, corp, key_word, start=None, end=None):
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        return dart_event.event(self.api_key, corp_code, key_word, start, end)

    # 6. 증권신고서 
    def regstate(self, corp, key_word, start=None, end=None):
        corp_code = self.find_corp_code(corp)
        if not corp_code:
            raise ValueError(f'could not find "{corp}"')
        return dart_regstate.regstate(self.api_key, corp_code, key_word, start, end)

    # 7. UTILS
    # utils: list_date 특정 날짜의 공시보고서 전체 (deprecated)
    def list_date(self, date=None, final=True, cache=True):
        warnings.warn('list_date() is deprecated. use list_date_ex()')
        
    # utils: list_date_ex 특정 날짜의 공시보고서 전체 데이터프레임 (시간포함)
    def list_date_ex(self, date=None, cache=True):
        return dart_utils.list_date_ex(date, cache=cache)
    
    # utils: attach document list: 첨부문서의 목록정보(제목, URL)을 데이터프레임
    def attach_docs(self, s, match=None):
        return dart_utils.attach_docs(s, match=match)

    # utils: attach document list: 첨부문서의 목록정보(제목, URL)을 데이터프레임
    def attach_doc_list(self, s, match=None):
        print('attach_doc_list() will deprecated. use attach_files() instead')
        return dart_utils.attach_docs(s, match=match)

    # utils: subdocument list: 하위 문서 목록정보(제목, URL)을 데이터프레임
    def sub_docs(self, s, match=None):
        return dart_utils.sub_docs(s, match=match)
    
    # utils: attach files file list: 첨부파일 목록정보를 데이터프레임
    def attach_files(self, s):
        return dart_utils.attach_files(s)

    # utils: attach files file list: 첨부파일 목록정보를 데이터프레임
    def attach_file_list(self, s):
        print('attach_file_list() will deprecated. use attach_files() instead')
        return dart_utils.attach_files(s)

    # utils: url 을 fn 으로 저장
    def download(self, url, fn):
        return dart_utils.download(url, fn)

    def retrieve(self, url, fn):
        print('retrieve() will deprecated. use download() instead')
        return dart_utils.download(url, fn)



#-*- coding:utf-8 -*-
# 2020-2023 FinanceData.KR http://financedata.kr fb.com/financedata

import requests
import json
import pandas as pd

def report(api_key, corp_code, key_word, bsns_year, reprt_code='11011'):
    key_word_map = {
        '조건부자본증권미상환': 'cndlCaplScritsNrdmpBlce', # 조건부 자본증권 미상환 잔액, ※ 2015년 이후 부터 제공
        '미등기임원보수': 'unrstExctvMendngSttus', # 미등기임원 보수현황
        '회사채미상환': 'cprndNrdmpBlce', # 회사채 미상환 잔액
        '단기사채미상환': 'srtpdPsndbtNrdmpBlce', # 단기사채 미상환 잔액
        '기업어음미상환': 'entrprsBilScritsNrdmpBlce', # 기업어음증권 미상환 잔액
        '채무증권발행': 'detScritsIsuAcmslt', # 채무증권 발행실적
        '사모자금사용': 'prvsrpCptalUseDtls', # 사모자금의 사용내역
        '공모자금사용': 'pssrpCptalUseDtls', # 공모자금의 사용내역
        '임원전체보수승인': 'drctrAdtAllMendngSttusGmtsckConfmAmount', # 이사·감사 전체의 보수현황(주주총회 승인금액)
        '임원전체보수유형': 'drctrAdtAllMendngSttusMendngPymntamtTyCl', # 이사·감사 전체의 보수현황(보수지급금액 - 유형별)
        '주식총수': 'stockTotqySttus', # 주식의 총수 현황
        '회계감사': 'accnutAdtorNmNdAdtOpinion', # 회계감사인의 명칭 및 감사의견
        '감사용역': 'adtServcCnclsSttus', # 감사용역체결현황
        '회계감사용역계약': 'accnutAdtorNonAdtServcCnclsSttus', # 회계감사인과의 비감사용역 계약체결 현황
        '사외이사': 'outcmpnyDrctrNdChangeSttus', # 사외이사 및 그 변동현황
        '신종자본증권미상환': 'newCaplScritsNrdmpBlce', # 신종자본증권 미상환 잔액
        '증자': 'irdsSttus', # 증자(감자) 현황
        '배당': 'alotMatter', # 배당에 관한 사항
        '자기주식': 'tesstkAcqsDspsSttus', # 자기주식 취득 및 처분 현황
        '최대주주': 'hyslrSttus', # 최대주주 현황
        '최대주주변동': 'hyslrChgSttus', # 최대주주 변동현황
        '소액주주': 'mrhlSttus', # 소액주주 현황
        '임원': 'exctvSttus', # 임원 현황
        '직원': 'empSttus', # 직원 현황
        '임원개인보수': 'hmvAuditIndvdlBySttus', # 이사·감사의 개인별 보수 현황
        '임원전체보수': 'hmvAuditAllSttus', # 이사·감사 전체의 보수현황
        '개인별보수': 'indvdlByPay', # 개인별 보수지급 금액(5억이상 상위5인)
        '타법인출자': 'otrCprInvstmntSttus', # 타법인 출자현황
    } 

    if key_word not in key_word_map.keys():
        raise ValueError('key_word is invalid: you can use one of ', key_word_map.keys())

    url = f"https://opendart.fss.or.kr/api/{key_word_map[key_word]}.json"
    params = {
        'crtfc_key': api_key,    # 인증키
        'corp_code': corp_code,   # 회사 고유번호
        'bsns_year': bsns_year,   # 사업년도
        'reprt_code': reprt_code, # 보고서 코드 ("11011"=사업보고서)
    }
    r = requests.get(url, params=params)
    jo = r.json()
    
    if jo['status'] != '000' or 'list' not in jo:
        print(jo)
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])



#-*- coding:utf-8 -*-
# 2020-2021 FinanceData.KR http://financedata.kr fb.com/financedata

import os
import re
import time
from datetime import datetime
from pandas import to_datetime
from urllib.parse import urlparse, parse_qs, quote_plus
import requests
from bs4 import BeautifulSoup
import pandas as pd
import json
import difflib

try:
    from pandas import json_normalize
except ImportError:
    from pandas.io.json import json_normalize
    
USER_AGENT = 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/78.0.3904.108 Safari/537.36'

def _validate_dates(start, end):
    start = to_datetime(start)
    end = to_datetime(end)

    if start is None:
        start = datetime(1970, 1, 1)
    if end is None:
        end = datetime.today()
    return start, end

def _requests_get_cache(url, headers=None):
    docs_cache_dir = 'docs_cache'
    if not os.path.exists(docs_cache_dir):
        os.makedirs(docs_cache_dir)
    
    fn = os.path.join(docs_cache_dir, quote_plus(url))
    if not os.path.isfile(fn) or os.path.getsize(fn) == 0:
        with open(fn, 'wt') as f:
            r = requests.get(url, headers=headers)
            f.write(r.text)
            xhtml_text = r.text
    else:
        with open(fn, 'rt') as f:
            xhtml_text = f.read()
            return xhtml_text
    return xhtml_text


def list_date_ex(date=None, cache=True):
    '''
    지정한 날짜의 보고서의 목록 전체를 데이터프레임으로 반환 합니다(시간 포함)
    * date: 조회일 (기본값: 당일)
    '''
    date = pd.to_datetime(date) if date else datetime.today() 
    date_str = date.strftime('%Y.%m.%d')

    columns = ['rcept_dt', 'corp_cls', 'corp_code', 'corp_name', 'rcept_no', 'report_nm', 'flr_nm', 'rm']
   
    df_list = []
    for page in range(1, 100):
        time.sleep(0.1)

        url = f'http://dart.fss.or.kr/dsac001/search.ax?selectDate={date_str}&pageGrouping=A&currentPage={page}'
        headers = {'User-Agent': USER_AGENT}
        xhtml_text = _requests_get_cache(url, headers=headers) if cache else requests.get(url, headers).text

        if '검색된 자료가 없습니다' in xhtml_text:
            if page == 1:
                return pd.DataFrame(columns=columns)
            break

        data_list = []
        soup = BeautifulSoup(xhtml_text, features='lxml')
        trs = soup.table.find_all('tr')
        for tr in trs[1:]:
            tds = tr.find_all('td')
            hhmm = tds[0].text.strip()
            corp_class = tds[1].span.span['title'].replace('시장', '') if tds[1].span.span else ''
            code = re.findall("openCorpInfoNew\('(.*?)', '", tds[1].span.a['href'])[0]
            name = tds[1].a.text.strip()
            rcp_no = tds[2].a['href'].split('=')[1]
            title = ' '.join(tds[2].a.text.split())
            fr_name = tds[3].text
            rcp_date = tds[4].text.replace('.', '-')
            remark = ' '.join([img['title'] for img in tds[5].find_all('img')])

            rm_str_list = [ # 비고에서 삭제할 문자열
                '본 공시사항은', 
                '소관임', 
                '본 보고서 제출 후',
                '가 있으니 관련 보고서를 참조하시기 바람',
                '본 보고서는',
                '을 포함한 것임',

            ]
            remark = re.sub('|'.join(rm_str_list), '', remark)
            remark = ' '.join(remark.split())
            dt = date.strftime('%Y-%m-%d') + ' ' + hhmm
            data_list.append([dt, corp_class, code, name, rcp_no, title, fr_name, remark])

        df = pd.DataFrame(data_list, columns=columns)
        df['rcept_dt'] = pd.to_datetime(df['rcept_dt'])
        df_list.append(df)
    return pd.concat(df_list)    


def attach_file_list(rcp_no): # rcp_no or URL
    '''
    접수번호(rcp_no)에 속한 첨부파일 목록정보를 데이터프레임으로 반환합니다.
    * rcp_no: 접수번호를 지정합니다. rcp_no 대신 첨부문서의 URL(http로 시작)을 사용할 수 도 있습니다.
    '''
    if rcp_no.startswith('http'):
        download_url = rcp_no
    else:
        url = f"http://dart.fss.or.kr/dsaf001/main.do?rcpNo={rcp_no}"
        r = requests.get(url, headers={'User-Agent': USER_AGENT})

        dcm_no = None
        matches = re.findall(f"openPdfDownload\('{rcp_no}', '(.*)'\)", r.text)
        if matches:
            dcm_no = matches[0]

        if not dcm_no:
            print(f'{url} 다운로드 페이지를 포함하고 있지 않습니다')

        download_url = f'http://dart.fss.or.kr/pdf/download/main.do?rcp_no={rcp_no}&dcm_no={dcm_no}'

    r = requests.get(download_url, headers={'User-Agent': USER_AGENT})
    soup = BeautifulSoup(r.text, features='lxml')
    table = soup.find('table')
    trs = table.find_all('tr')

    row_list = []
    for tr in trs[1:]:
        tds = tr.find_all('td')
        fname = tds[0].text
        flink = 'http://dart.fss.or.kr' + tds[1].a['href']
        matches = re.findall('/pdf/download/(.*).do', flink)
        ftype = matches[0] if matches else None
        row_list.append([fname, flink, ftype])

    return pd.DataFrame(row_list, columns=['file_name', 'url', 'type'])


def attach_doc_list(rcp_no, match=None):
    '''
    첨부문서의 목록정보(title, url)을 데이터프레임으로 반환합니다. match를 지정하면 지정한 문자열과 가장 유사한 순서로 소트하여 데이터프레임을 반환 합니다.
    * rcp_no: 접수번호
    * match: 문서 제목과 가장 유사한 순서로 소트
    '''
    r = requests.get(f'http://dart.fss.or.kr/dsaf001/main.do?rcpNo={rcp_no}', headers={'User-Agent': USER_AGENT})
    soup = BeautifulSoup(r.text, features='lxml')

    row_list = []
    att = soup.find(id='att')
    if not att:
        raise Exception(f'rcp_no={rcp_no} 첨부문서를 포함하고 있지 않습니다')

    for opt in att.find_all('option'):
        if opt['value'] == 'null':
            continue
        title = ' '.join(opt.text.split())
        url = f'http://dart.fss.or.kr/dsaf001/main.do?{opt["value"]}'
        row_list.append([title, url])
        
    df = pd.DataFrame(row_list, columns=['title', 'url'])
    if match:
        df['similarity'] = df.title.apply(lambda x: difflib.SequenceMatcher(None, x, match).ratio())
        df = df.sort_values('similarity', ascending=False)
    return df[['title', 'url']].copy()


pattern = '''\t+node[12]\['text'] =[ ]*"(.*?)";
\t+node[12]\['id'\] = "(.*?)";
\t+node[12]\['rcpNo'\] = "(.*?)";
\t+node[12]\['dcmNo'\] = "(.*?)";
\t+node[12]\['eleId'\] = "(.*?)";
\t+node[12]\['offset'\] = "(.*?)";
\t+node[12]\['length'\] = "(.*?)";
\t+node[12]\['dtd'\] = "(.*?)";
\t+node[12]\['tocNo'\] =  "(.*?)";'''

def sub_docs(rcp_no, match=None):
    '''
    지정한 URL문서에 속해있는 하위 문서 목록정보(title, url)을 데이터프레임으로 반환합니다
    * rcp_no: 접수번호를 지정합니다. rcp_no 대신 첨부문서의 URL(http로 시작)을 사용할 수 도 있습니다.
    * match: 매칭할 문자열 (문자열을 지정하면 문서 제목과 가장 유사한 순서로 소트 합니다)
    '''
    if rcp_no.isdecimal():
        r = requests.get(f'http://dart.fss.or.kr/dsaf001/main.do?rcpNo={rcp_no}', headers={'User-Agent': USER_AGENT})
    elif rcp_no.startswith('http'):
        r = requests.get(rcp_no, headers={'User-Agent': USER_AGENT})
    else:
        raise ValueError('invalid `rcp_no`(or url)')
        
    ## 하위 문서 URL 추출
    doc_urls = []
    doc_titles = []
    multi_page_matches = re.findall(pattern, r.text)
    single_page_matches = re.findall('''viewDoc\('(.*?)', '(.*?)', '(.*?)', '(.*?)', '(.*?)', '(.*?)',''\);''', r.text)
    if  multi_page_matches: 
        for m in multi_page_matches:
            params = f'rcpNo={m[2]}&dcmNo={m[3]}&eleId={m[4]}&offset={m[5]}&length={m[6]}&dtd={m[7]}'
            url = f'http://dart.fss.or.kr/report/viewer.do?{params}'
            doc_urls.append(url)
            doc_titles.append(m[0])
    elif single_page_matches:
        m = single_page_matches[0]
        doc_titles.append(BeautifulSoup(r.text).find('title').text.strip())
        doc_urls.append(f'http://dart.fss.or.kr/report/viewer.do?rcpNo={m[0]}&dcmNo={m[1]}&eleId={m[2]}&offset={m[3]}&length={m[4]}&dtd={m[5]}')
    else:
        raise Exception(f'http://dart.fss.or.kr/dsaf001/main.do?rcpNo={rcp_no} 하위 페이지를 포함하고 있지 않습니다')

    df = pd.DataFrame(zip(doc_titles, doc_urls), columns=['title', 'url'])
    if match:
        df['similarity'] = df.title.apply(lambda x: difflib.SequenceMatcher(None, x, match).ratio())
        df = df.sort_values('similarity', ascending=False)
    return df[['title', 'url']].copy()


def retrieve(url, fn=None):
    fn = fn if fn else url.split('/')[-1]
    r = requests.get(url, stream=True, headers={'User-Agent': USER_AGENT})
    if r.status_code != 200:
        print(r.status_code)
        return None

    with open(fn, "wb") as f:
        for chunk in r.iter_content(chunk_size=4096):
            f.write(chunk) if chunk else None
    return fn



import requests
import json
from pandas.io.json import json_normalize

# 대량보유 상황보고
def major_shareholders(api_key, corp_code):
    url = 'https://opendart.fss.or.kr/api/majorstock.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
    }

    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    return json_normalize(jo, 'list')

# 임원ㆍ주요주주 소유보고
def major_shareholders_exec(api_key, corp_code):
    url = 'https://opendart.fss.or.kr/api/elestock.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
    }

    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    if 'list' not in jo:
        return None
    return json_normalize(jo, 'list')



#-*- coding:utf-8 -*-
# 2020-2023 FinanceData.KR http://financedata.kr fb.com/financedata

import requests
import pandas as pd
from datetime import datetime, timedelta

def regstate(api_key, corp_code, key_word, start=None, end=None):
    start = pd.to_datetime(start) if start else pd.to_datetime('1900-01-01')
    end = pd.to_datetime(end) if end else datetime.today()

    key_word_map = {
        '주식의포괄적교환이전': 'extrRs', # 주식의포괄적교환이전
        '합병': 'mgRs', # 합병
        '증권예탁증권': 'stkdpRs', # 증권예탁증권
        '채무증권': 'bdRs', #  채무증권
        '지분증권': 'estkRs', # 지분증권 
        '분할': 'dvRs', #  분할
    } 

    if key_word not in key_word_map.keys():
        raise ValueError('key_word is invalid: you can use one of ', key_word_map.keys())

    url = f"https://opendart.fss.or.kr/api/{key_word_map[key_word]}.json"
    params = {
        'crtfc_key': api_key,    # 인증키
        'corp_code': corp_code,   # 회사 고유번호
        'bgn_de': start.strftime('%Y%m%d'),  # 시작일(최초접수일)
        'end_de': end.strftime('%Y%m%d'), # 종료일(최초접수일)
    }
    r = requests.get(url, params=params)
    jo = r.json()
    
    if jo['status'] != '000':
        print(jo)
        return pd.DataFrame()
    elif 'list' in jo:
        return pd.DataFrame(jo['list'])
    elif 'group' in jo:
        group = pd.DataFrame(jo['group'])
        df_list = []
        for ix, row in group.iterrows():
            inner_df = pd.DataFrame(row['list'])
            inner_df['title'] = row['title']
            df_list.append(inner_df)
        return pd.concat(df_list)
    else:
        print('invalid respose')
        print(jo)
    return pd.DataFrame()





from setuptools import setup, find_packages

setup(
    name='yooncloud-dart',
    version='0.0.2',
    install_requires=["numpy==1.26.4", "pandas==2.2.1"],
    packages=find_packages(),
    python_requires='>=3.8',
)


import io
import zipfile
import requests
import json
from pandas.io.json import json_normalize

def finstate(api_key, corp_code, bsns_year, reprt_code='11011'):
    url = 'https://opendart.fss.or.kr/api/fnlttSinglAcnt.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
        'bsns_year':  bsns_year,   # 사업년도
        'reprt_code': reprt_code, # "11011": 사업보고서
    }

    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    if 'list' not in jo:
        return None
    return json_normalize(jo, 'list')



#-*- coding:utf-8 -*-
# 2022 FinanceData.KR http://financedata.kr fb.com/financedata

import requests
import json
import pandas as pd
from datetime import datetime, timedelta

def event(api_key, corp_code, key_word, start=None, end=None):
    start = pd.to_datetime(start) if start else pd.to_datetime('1900-01-01')
    end = pd.to_datetime(end) if end else datetime.today()

    key_word_map = {
        '부도발생': 'dfOcr', # 부도발생 
        '영업정지': 'bsnSp', # 영업정지 
        '회생절차': 'ctrcvsBgrq', #  회생절차 개시신청
        '해산사유': 'dsRsOcr', #  해산사유 발생 
        '유상증자': 'piicDecsn', #  유상증자 결정  
        '무상증자': 'fricDecsn', #  무상증자 결정 
        '유무상증자': 'pifricDecsn', #  유무상증자 결정
        '감자': 'crDecsn', #  감자 결정 
        '관리절차개시': 'bnkMngtPcbg', #  채권은행 등의 관리절차 개시 
        '소송': 'lwstLg', #  소송 등의 제기
        '해외상장결정': 'ovLstDecsn', #  해외 증권시장 주권등 상장 결정
        '해외상장폐지결정': 'ovDlstDecsn', #  해외 증권시장 주권등 상장폐지 결정
        '해외상장': 'ovLst', #  해외 증권시장 주권등 상장
        '해외상장폐지': 'ovDlst', #  해외 증권시장 주권등 상장폐지
        '전환사채발행': 'cvbdIsDecsn', #  전환사채권 발행결정
        '신주인수권부사채발행': 'bdwtIsDecsn', #  신주인수권부사채권 발행결정
        '교환사채발행': 'exbdIsDecsn', #  교환사채권 발행결정
        '관리절차중단': 'bnkMngtPcsp', #  채권은행 등의 관리절차 중단
        '조건부자본증권발행': 'wdCocobdIsDecsn', #  상각형 조건부자본증권 발행결정
        '자산양수도': 'astInhtrfEtcPtbkOpt', #  자산양수도(기타), 풋백옵션
        '타법인증권양도': 'otcprStkInvscrTrfDecsn', #  타법인 주식 및 출자 증권 양도결정
        '유형자산양도': 'tgastTrfDecsn', #  유형자산 양도 결정 
        '유형자산양수': 'tgastInhDecsn', #  유형자산 양수 결정
        '타법인증권양수': 'otcprStkInvscrInhDecsn', #  타법인 주식 및 출자증권 양수결정
        '영업양도': 'bsnTrfDecsn', #  영업양도 결정 
        '영업양수': 'bsnInhDecsn', #  영업양수 결정
        '자기주식취득신탁계약해지': 'tsstkAqTrctrCcDecsn', #  자기주식취득 신탁계약 해지 결정
        '자기주식취득신탁계약체결': 'tsstkAqTrctrCnsDecsn', #  자기주식취득 신탁계약 체결 결정
        '자기주식처분': 'tsstkDpDecsn', #  자기주식 처분 결정
        '자기주식취득': 'tsstkAqDecsn', #  자기주식 취득 결정
        '주식교환': 'stkExtrDecsn', #  주식교환·이전 결정
        '회사분할합병': 'cmpDvmgDecsn', #  회사분할합병 결정
        '회사분할': 'cmpDvDecsn', #  회사분할 결정
        '회사합병': 'cmpMgDecsn', #  회사합병 결정
        '사채권양수': 'stkrtbdInhDecsn', #  주권 관련 사채권 양수 결정
        '사채권양도결정': 'stkrtbdTrfDecsn', #  주권 관련 사채권 양도 결정
    } 

    if key_word not in key_word_map.keys():
        raise ValueError('key_word is invalid: you can use one of ', key_word_map.keys())

    url = f"https://opendart.fss.or.kr/api/{key_word_map[key_word]}.json"
    params = {
        'crtfc_key': api_key,    # 인증키
        'corp_code': corp_code,   # 회사 고유번호
        'bgn_de': start.strftime('%Y%m%d'),  # 시작일(최초접수일)
        'end_de': end.strftime('%Y%m%d'), # 종료일(최초접수일)
    }
    r = requests.get(url, params=params)
    jo = r.json()
    
    if jo['status'] != '000' or 'list' not in jo:
        print(jo)
        return pd.DataFrame()
    return pd.DataFrame(jo['list'])



import requests
import json
from pandas.io.json import json_normalize

def report(api_key, corp_code, key_word, bsns_year, reprt_code='11011'):
    key_word_map = {
        '증자': 'irdsSttus',
        '배당': 'alotMatter',
        '자기주식': 'tesstkAcqsDspsSttus',
        '최대주주': 'hyslrSttus',
        '최대주주변동': 'hyslrChgSttus',
        '소액주주': 'mrhlSttus',
        '임원': 'exctvSttus',
        '직원': 'empSttus',
        '임원개인보수': 'hmvAuditIndvdlBySttus',
        '임원전체보수': 'hmvAuditAllSttus',
        '개인별보수': 'indvdlByPay',
        '타법인출자': 'otrCprInvstmntSttus',
    }

    if key_word not in key_word_map.keys():
        raise ValueError('key_word is invalid: you can use one of ', key_word_map.keys())

    url = "https://opendart.fss.or.kr/api/{}.json".format(key_word_map[key_word])
    params = {
        'crtfc_key': api_key,    # 인증키
        'corp_code': corp_code,   # 회사 고유번호
        'bsns_year': bsns_year,   # 사업년도
        'reprt_code': reprt_code, # 보고서 코드 ("11011"=사업보고서)
    }
    r = requests.get(url, params=params)
    jo = json.loads(r.text)
    
    if 'list' not in jo:
        return None
    return json_normalize(jo, 'list')



#-*- coding:utf-8 -*-
# 2020 FinanceData.KR http://financedata.kr fb.com/financedata

import requests
import zipfile
import io
import os
import json
import pandas as pd
import xml.etree.ElementTree as ET
from datetime import datetime, timedelta

try:
    from pandas import json_normalize
except ImportError:
    from pandas.io.json import json_normalize

# 1. 공시정보 - 공시검색(목록)
def list(api_key, corp_code='', start=None, end=None, kind='', kind_detail='', final=False):
    start = pd.to_datetime(start).strftime('%Y%m%d') if start else '19000101'
    end = pd.to_datetime(end).strftime('%Y%m%d') if end else datetime.today().strftime('%Y%m%d')
    
    url = 'https://opendart.fss.or.kr/api/list.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
        'bgn_de': start,
        'end_de': end,
        'last_reprt_at': 'Y' if final else 'N', # 최종보고서 여부
        'page_no': 1,
        'page_count': 100,
    }
    if kind:
        params['pblntf_ty'] = kind # 공시유형: 기본값 'A'=정기공시
    if kind_detail:
        params['pblntf_detail_ty'] = kind_detail

    r = requests.get(url, params=params)
    try:
        tree = ET.XML(r.content)
        status = tree.find('status').text
        message = tree.find('message').text
        if status != '000':
            raise ValueError({'status': status, 'message': message})
    except ET.ParseError as e:
        jo = json.loads(r.text)
        if jo['status'] != '000':
            print(ValueError(r.text))

    jo = json.loads(r.text)
    if 'list' not in jo:
        return pd.DataFrame()
    df = json_normalize(jo, 'list')

    # paging
    for page in range(2, jo['total_page']+1):
        params['page_no'] = page
        r = requests.get(url, params=params)
        jo = json.loads(r.text)
        df = df.append(json_normalize(jo, 'list'))
    return df


# 1-2. 공시정보 - 기업개황
def company(api_key, corp_code):
    url = 'https://opendart.fss.or.kr/api/company.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': corp_code,
    }
    r = requests.get(url, params=params)
    try:
        tree = ET.XML(r.content)
        status = tree.find('status').text,
        message = tree.find('message').text
        if status != '000':
            raise ValueError({'status': status, 'message': message})
    except ET.ParseError as e:
        pass
    return json.loads(r.text)

# 1-2. 공시정보 - 기업개황: 지정된 이름(name)을 포함하는 회사들의 corp_code 리스트를 반환
def company_by_name(api_key, corp_code_list):
    url = 'https://opendart.fss.or.kr/api/company.json'
    params = {
        'crtfc_key': api_key,
        'corp_code': '',
    }
    company_list = []
    for corp_code in corp_code_list:
        params['corp_code'] = corp_code
        r = requests.get(url, params=params)
        company_list.append(json.loads(r.text))
    return company_list

# 1-3. 공시정보 - 공시서류원본파일
def document(api_key, rcp_no, cache=True):
    # create cache directory if not exists
    docs_cache_dir = 'docs_cache'
    if not os.path.exists(docs_cache_dir):
        os.makedirs(docs_cache_dir)

    # read and return document if exists
    fn = os.path.join(docs_cache_dir, 'dart-{}.xhml'.format(rcp_no))
    if os.path.isfile(fn) and os.path.getsize(fn) > 0:
        try:
            with open(fn, 'r') as f:
                return f.read()
        except UnicodeDecodeError as e:
            with open(fn, 'r', encoding="utf-8") as f:
                return f.read()

    url = 'https://opendart.fss.or.kr/api/document.xml'
    params = {
        'crtfc_key': api_key,
        'rcept_no': rcp_no,
    }

    r = requests.get(url, params=params)
    try:
        tree = ET.XML(r.content)
        status = tree.find('status').text
        message = tree.find('message').text
        if status != '000':
            raise ValueError({'status': status, 'message': message})
    except ET.ParseError as e:
        pass
    zf = zipfile.ZipFile(io.BytesIO(r.content))
    info_list = zf.infolist()
    xml_data = zf.read(info_list[0].filename)

    try:
        xml_text = xml_data.decode('euc-kr')
    except UnicodeDecodeError as e:
        xml_text = xml_data.decode('utf-8')
    except UnicodeDecodeError as e:
        xml_text = xml_data

    # save document to cache 
    with open(fn, 'wt') as f:
        f.write(xml_text)
    return xml_text


# 1-4 고유번호: api/corpCode.xml
def corp_codes(api_key):
        url = 'https://opendart.fss.or.kr/api/corpCode.xml'
        params = { 'crtfc_key': api_key, }

        r = requests.get(url, params=params)
        try:
            tree = ET.XML(r.content)
            status = tree.find('status').text
            message = tree.find('message').text
            if status != '000':
                raise ValueError({'status': status, 'message': message})
        except ET.ParseError as e:
            pass

        zf = zipfile.ZipFile(io.BytesIO(r.content))
        xml_data = zf.read('CORPCODE.xml')

        # XML to DataFrame
        tree = ET.XML(xml_data)
        all_records = []

        element = tree.findall('list')
        for i, child in enumerate(element):
            record = {}
            for i, subchild in enumerate(child):
                record[subchild.tag] = subchild.text
            all_records.append(record)
        return pd.DataFrame(all_records)



#-*- coding:utf-8 -*-
# 2020-2023 FinanceData.KR http://financedata.kr fb.com/financedata
import sys
from .dart import *

__version__ = '0.0.2'
__all__ = ['__version__', 'OpenDartReader']

sys.modules['yooncloud_dart'] = dart.OpenDartReader
